@@ -5,14 +5,14 @@
     :password :env/clojars_jenkins_password
     :sign-releases false})
 
-(defproject puppetlabs/ssl-utils "3.5.0"
+(defproject puppetlabs/ssl-utils "3.5.3"
   :url "http://www.github.com/puppetlabs/jvm-ssl-utils"
 
   :description "SSL certificate management on the JVM."
 
   :min-lein-version "2.9.10"
 
-  :parent-project {:coords [puppetlabs/clj-parent "5.2.6"]
+  :parent-project {:coords [puppetlabs/clj-parent "5.6.6"]
                    :inherit [:managed-dependencies]}
 
   ;; Abort when version ranges or version conflicts are detected in
@@ -60,6 +60,7 @@
                                        ["-Djava.security.properties==jdk8-fips-security"]
                                        (throw unsupported-ex))
                                    11 ["-Djava.security.properties==jdk11-fips-security"]
+                                   17 ["-Djava.security.properties==jdk17-fips-security"]
                                    (throw unsupported-ex)))
                     :resource-paths ["test-resources"]}
 
@@ -67,8 +68,13 @@
                            :jar-exclusions ^:replace []
                            :source-paths ^:replace ["src/clojure" "src/java"]}}
 
-  :plugins [[lein-parent "0.3.7"]
-            [puppetlabs/i18n "0.8.0"]]
+  :plugins [[lein-parent "0.3.9"]
+            [puppetlabs/i18n "0.9.2"]
+            [jonase/eastwood "1.2.2" :exclusions [org.clojure/clojure]]]
+
+  :eastwood {:exclude-linters [:no-ns-form-found :reflection]
+             :continue-on-exception true}
+
   :lein-release {:scm         :git
                  :deploy-via  :lein-deploy}
   :deploy-repositories [["releases" ~(deploy-info "https://clojars.org/repo")]
