@@ -12,7 +12,6 @@ import org.bouncycastle.asn1.ASN1TaggedObject;
 import org.bouncycastle.asn1.DERBitString;
 import org.bouncycastle.asn1.DERIA5String;
 import org.bouncycastle.asn1.DEROctetString;
-import org.bouncycastle.asn1.DERSequence;
 import org.bouncycastle.asn1.DERUTF8String;
 import org.bouncycastle.asn1.misc.MiscObjectIdentifiers;
 import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
@@ -41,7 +40,6 @@ import org.bouncycastle.operator.OperatorCreationException;
 import org.bouncycastle.operator.jcajce.JcaDigestCalculatorProviderBuilder;
 import org.bouncycastle.pkcs.PKCS10CertificationRequest;
 
-import java.io.EOFException;
 import java.io.IOException;
 import java.math.BigInteger;
 import java.net.InetAddress;
@@ -64,7 +62,14 @@ import java.util.Set;
  * Utilities for working with X509 extensions.
  */
 public class ExtensionsUtils {
+    public static class AttributeDescriptor {
+        public String oid;
+        public ArrayList<Object> values;
 
+        AttributeDescriptor(){
+            this.values = new ArrayList<Object>();
+        }
+    }
     /**
      * CRLNumber OID 2.5.29.20
      */
@@ -187,6 +192,21 @@ public class ExtensionsUtils {
         }
     }
 
+    /**
+     * Given a bouncy Castle Certification Request, extract the attributes from that
+     * request
+     * @param csr - Bouncy Castle certification request
+     * @return an array of attributes in the CSR
+     */
+    public static AttributeDescriptor[] getAttributesList(PKCS10CertificationRequest csr) throws IOException {
+        Attribute[] attr = csr.getAttributes();
+        AttributeDescriptor[] result = new AttributeDescriptor[(attr.length)];
+        for (int i = 0; i < attr.length; i++){
+            result[i] = makeAttributeDescriptor(attr[i]);
+        }
+        return result;
+    }
+
     /**
      * Given a Java certificate, get a map containing the value
      * and criticality of the extensions described by the given OID. If the OID
@@ -536,6 +556,15 @@ public class ExtensionsUtils {
         }
     }
 
+    private static AttributeDescriptor makeAttributeDescriptor(Attribute attr) throws IOException {
+        AttributeDescriptor result = new AttributeDescriptor();
+        result.oid = attr.getAttrType().getId();
+        for (ASN1Encodable attributeValue : attr.getAttributeValues()) {
+            result.values.add(asn1ObjToObj(attributeValue));
+        }
+        return result;
+    }
+
     /**
      * Convert a chunk of binary data into the Bouncy Castle ASN1 data structure
      * which represents the data it contains accord to its OID. I've searched
@@ -578,8 +607,8 @@ public class ExtensionsUtils {
             return CRLNumber.getInstance(data);
         } else {
             try {
-                // If the oid is unknown, attempt to parse it as a UTF8 String
-                return DERUTF8String.getInstance(data);
+                // If the oid is unknown, use the base primitive conversion
+                return ASN1Primitive.fromByteArray(data);
             } catch (Exception e) {
                 // This is required to maintain backwards compatibility with
                 // the erroneous method that Puppet previously used to sign
@@ -627,17 +656,13 @@ public class ExtensionsUtils {
         } else if (asn1Prim instanceof DERBitString) {
             DERBitString bitString = (DERBitString)asn1Prim;
             return bitString.getString();
-        } else if (asn1Prim instanceof ASN1TaggedObject) {
-            ASN1TaggedObject taggedObj = (ASN1TaggedObject)asn1Prim;
-            return asn1ObjToObj(taggedObj.getObject());
         } else if (asn1Prim instanceof ASN1Sequence) {
             return asn1SeqToList((ASN1Sequence) asn1Prim);
         } else if (asn1Prim instanceof ASN1String) {
             ASN1String str = (ASN1String)asn1Prim;
             return str.getString();
         } else if (asn1Prim instanceof ASN1OctetString) {
-            ASN1OctetString str = (ASN1OctetString) asn1Prim;
-            return new String(str.getOctets(), "UTF-8");
+            return ASN1Primitive.fromByteArray(((ASN1OctetString) asn1Prim).getOctets()).toString();
         } else if (asn1Prim instanceof X500Name) {
             X500Name name = (X500Name) asn1Prim;
             return name.toString();
@@ -748,7 +773,7 @@ public class ExtensionsUtils {
                 .get(new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1));
 
         X509ExtensionUtils utils = new JcaX509ExtensionUtils(digCalc);
-        if (truncate == true) {
+        if (truncate) {
             return utils.createTruncatedSubjectKeyIdentifier(pubKeyInfo);
         }
         return utils.createSubjectKeyIdentifier(pubKeyInfo);
@@ -769,7 +794,7 @@ public class ExtensionsUtils {
             SubjectPublicKeyInfo.getInstance(pubKey.getEncoded());
 
         JcaX509ExtensionUtils utils = extensionUtils();
-        if (truncateKey == true) {
+        if (truncateKey) {
             byte[] shortKey = utils.createTruncatedSubjectKeyIdentifier(
                                       authPubKeyInfo).getKeyIdentifier();
             return new AuthorityKeyIdentifier(shortKey);
