@@ -3,8 +3,7 @@ package com.puppetlabs.ssl_utils;
 import org.apache.commons.codec.digest.DigestUtils;
 import org.apache.commons.codec.binary.Hex;
 
-import org.bouncycastle.asn1.ASN1ObjectIdentifier;
-import org.bouncycastle.asn1.DERNull;
+import org.bouncycastle.asn1.*;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PrivateKeyInfo;
 import org.bouncycastle.asn1.pkcs.RSAPublicKey;
@@ -108,6 +107,8 @@ public class SSLUtils {
     public static final String BOUNCYCASTLE_JSSE_PROVIDER = "BCJSSE";
     public static final String TLS_PROTOCOL = "TLS";
 
+    private static int crlLifetimeSeconds = 5 * 365 * 24 * 60 * 60;  // default to 5 years
+
     public static boolean isFIPS() {
         try {
             return getProviderClass().getCanonicalName().equals(FIPS_PROVIDER_CLASS);
@@ -183,13 +184,14 @@ public class SSLUtils {
      * @param keyPair The subject's public and private keys.
      * @param subjectDN The subject's CN.
      * @param extensions Extensions to add to this cert request.
+     * @param attributes to add to this cert request (converted to strings)
      * @return A request to certify the provided subject
      * @throws IOException
      * @throws OperatorCreationException
      * @see #generateKeyPair
      */
     public static PKCS10CertificationRequest generateCertificateRequest(KeyPair keyPair, String subjectDN,
-        List<Map<String, Object>> extensions)
+        List<Map<String, Object>> extensions, List<Map<String, Object>> attributes)
         throws IOException, OperatorCreationException, CertificateEncodingException
     {
         // TODO: the puppet code sets a property "version=0" on the request object
@@ -202,6 +204,31 @@ public class SSLUtils {
             requestBuilder.addAttribute(PKCSObjectIdentifiers.pkcs_9_at_extensionRequest, parsedExts);
         }
 
+        if ((attributes != null) && (attributes.size() > 0)) {
+            HashMap<String, ArrayList<ASN1Encodable>> mappedEntries = new HashMap<String, ArrayList<ASN1Encodable>>();
+            for (Map<String, Object> attributeMap : attributes) {
+                String oidString = (String)attributeMap.get("oid");
+                String value = attributeMap.get("value").toString();
+                DEROctetString convertedValue = new DEROctetString(
+                        new DERUTF8String(value));
+                if (mappedEntries.containsKey(oidString)) {
+                    mappedEntries.get(oidString).add(convertedValue);
+                } else {
+                    ArrayList<ASN1Encodable> newValues = new ArrayList<ASN1Encodable>();
+                    newValues.add(convertedValue);
+                    mappedEntries.put(oidString, newValues);
+                }
+
+            }
+            for (Map.Entry<String, ArrayList<ASN1Encodable>> entry: mappedEntries.entrySet()) {
+                ASN1ObjectIdentifier oid = new ASN1ObjectIdentifier(entry.getKey());
+                ArrayList<ASN1Encodable> values = entry.getValue();
+                ASN1Encodable[] convertedValues = new ASN1Encodable[entry.getValue().size()];
+                values.toArray(convertedValues);
+                requestBuilder.addAttribute(oid, convertedValues);
+            }
+        }
+
         return requestBuilder.build(
                 new JcaContentSignerBuilder("SHA1withRSA").
                         build(keyPair.getPrivate()));
@@ -333,7 +360,7 @@ public class SSLUtils {
     {
         DateTime now = DateTime.now();
         Date thisUpdate = now.toDate();
-        Date nextUpdate = now.plusYears(5).toDate();
+        Date nextUpdate = now.plusSeconds(crlLifetimeSeconds).toDate();
         return generateCRL(issuer, issuerPrivateKey, issuerPublicKey,
                            thisUpdate, nextUpdate, BigInteger.ZERO, null);
     }
@@ -1493,4 +1520,20 @@ public class SSLUtils {
         MessageDigest digest = DigestUtils.getDigest(digestAlgorithm);
         return Hex.encodeHexString(digest.digest(bytes));
     }
+
+    /***
+     * Get the number of seconds between when a CRL is generated and when it will expire.
+     * @return
+     */
+    public static int getCrlLifetimeSeconds() {
+        return crlLifetimeSeconds;
+    }
+
+    /***
+     * set the number of seconds after the generation time for a CRL before it will expire.
+     * @param seconds
+     */
+    public static void setCrlLifetimeSeconds(int seconds) {
+        crlLifetimeSeconds = seconds;
+    }
 }
